@@ -32,6 +32,8 @@
 #define OnigUChar unsigned char
 #endif
 
+#include "saphire_inner.h"
+
 typedef struct {
     string_obj* mName;
 
@@ -121,9 +123,9 @@ static BOOL read_one_argument(char* p, string_obj* buf)
     static unsigned char table[] = { 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-        1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 
-        1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+        1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0 
@@ -752,6 +754,272 @@ char* readline_method_completion(const char* text, int stat)
     return NULL;
 }
 
+char* readline_sweep_completion(const char* text, int stat)
+{
+    static hash_it* it;
+    static hash_it* it2;
+    static hash_it* it3;
+    static hash_it* it4;
+    static hash_it* it5;
+    static hash_it* it6;
+    static hash_it* it7;
+    static int wordlen;
+
+    if(stat == 0) {
+        wordlen = strlen(text);
+        it = hash_loop_begin(gArrays);
+        it2 = hash_loop_begin(gHashs);
+        it3 = hash_loop_begin(gGlobals);
+        it4 = hash_loop_begin(gFuncs);
+        it5 = hash_loop_begin(gClasses);
+        it6 = hash_loop_begin(gObjects);
+        it7 = hash_loop_begin(gRefs);
+    }
+
+    while(it) {
+        char* key = hash_loop_key(it);
+        it = hash_loop_next(it);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    while(it2) {
+        char* key = hash_loop_key(it2);
+        it2 = hash_loop_next(it2);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    while(it3) {
+        char* key = hash_loop_key(it3);
+        it3 = hash_loop_next(it3);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    while(it4) {
+        char* key = hash_loop_key(it4);
+        it4 = hash_loop_next(it4);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    while(it5) {
+        char* key = hash_loop_key(it5);
+        it5 = hash_loop_next(it5);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    while(it6) {
+        char* key = hash_loop_key(it6);
+        it6 = hash_loop_next(it6);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    while(it7) {
+        char* key = hash_loop_key(it7);
+        it7 = hash_loop_next(it7);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    return NULL;
+}
+
+char* readline_ary_completion(const char* text, int stat)
+{
+    static hash_it* it;
+    static int wordlen;
+
+    if(stat == 0) {
+        wordlen = strlen(text);
+        it = hash_loop_begin(gArrays);
+    }
+
+    while(it) {
+        char* key = hash_loop_key(it);
+        it = hash_loop_next(it);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    return NULL;
+}
+
+char* readline_hash_completion(const char* text, int stat)
+{
+    static hash_it* it;
+    static int wordlen;
+
+    if(stat == 0) {
+        wordlen = strlen(text);
+        it = hash_loop_begin(gHashs);
+    }
+
+    while(it) {
+        char* key = hash_loop_key(it);
+        it = hash_loop_next(it);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    return NULL;
+}
+
+char* readline_global_completion(const char* text, int stat)
+{
+    static hash_it* it;
+    static int wordlen;
+
+    if(stat == 0) {
+        wordlen = strlen(text);
+        it = hash_loop_begin(gGlobals);
+    }
+
+    while(it) {
+        char* key = hash_loop_key(it);
+        it = hash_loop_next(it);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    return NULL;
+}
+
+char* readline_def_completion(const char* text, int stat)
+{
+    static hash_it* it;
+    static int wordlen;
+
+    if(stat == 0) {
+        wordlen = strlen(text);
+        it = hash_loop_begin(gFuncs);
+    }
+
+    while(it) {
+        char* key = hash_loop_key(it);
+        it = hash_loop_next(it);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    return NULL;
+}
+
+char* readline_class_completion(const char* text, int stat)
+{
+    static hash_it* it;
+    static int wordlen;
+
+    if(stat == 0) {
+        wordlen = strlen(text);
+        it = hash_loop_begin(gClasses);
+    }
+
+    while(it) {
+        char* key = hash_loop_key(it);
+        it = hash_loop_next(it);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    return NULL;
+}
+
+char* readline_object_completion(const char* text, int stat)
+{
+    static hash_it* it;
+    static int wordlen;
+
+    if(stat == 0) {
+        wordlen = strlen(text);
+        it = hash_loop_begin(gObjects);
+    }
+
+    while(it) {
+        char* key = hash_loop_key(it);
+        it = hash_loop_next(it);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    return NULL;
+}
+
+char* readline_deref_completion(const char* text, int stat)
+{
+    static hash_it* it;
+    static int wordlen;
+
+    if(stat == 0) {
+        wordlen = strlen(text);
+        it = hash_loop_begin(gRefs);
+    }
+
+    while(it) {
+        char* key = hash_loop_key(it);
+        it = hash_loop_next(it);
+
+        if(!strncmp(text + 1, key, wordlen-1)) {
+            char buf[1024];
+            snprintf(buf, 1024, "$%s", key);
+            return strdup(buf);
+        }
+    }
+
+    return NULL;
+}
+
+char* readline_ref_completion(const char* text, int stat)
+{
+    static hash_it* it;
+    static int wordlen;
+
+    if(stat == 0) {
+        wordlen = strlen(text);
+        it = hash_loop_begin(gRefs);
+    }
+
+    while(it) {
+        char* key = hash_loop_key(it);
+        it = hash_loop_next(it);
+
+        if(!strncmp(text, key, wordlen)) {
+            return strdup(key);
+        }
+    }
+
+    return NULL;
+}
+
 char* readline_file_completion(const char* text, int rstat)
 {
     /// クォートを除外 ///
@@ -1006,7 +1274,9 @@ char** readline_on_complete(const char* text, int start, int end)
 {
     BOOL program = TRUE;
     BOOL user = FALSE;
+    BOOL ref = FALSE;
     BOOL method = FALSE;
+    string_obj* command = STRING_NEW("");
 
     string_obj* buf = STRING_NEW("");
 
@@ -1023,24 +1293,28 @@ char** readline_on_complete(const char* text, int start, int end)
         if(*p == '\\' && *(p+1) == 't') {
             p+=2;
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
         else if(*p == '\\' && *(p+1) == 'n') {
             p+=2;
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
         else if(*p == '\\' && *(p+1) == 'r') {
             p+=2;
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
         else if(*p == '\\') {
             p+=2;
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1058,6 +1332,7 @@ char** readline_on_complete(const char* text, int start, int end)
             p++;
             dquote = !dquote;
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1066,11 +1341,13 @@ char** readline_on_complete(const char* text, int start, int end)
             p+=3;
 
             program = TRUE;
+            string_put(command, "");
 
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1079,10 +1356,12 @@ char** readline_on_complete(const char* text, int start, int end)
             p+=2;
 
             program = TRUE;
+            string_put(command, "");
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1093,47 +1372,27 @@ char** readline_on_complete(const char* text, int start, int end)
                 p++;
             }
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
 
-        // 環境変数 $$
-        else if(!squote && *p == '$' && *(p+1) == '$') {
-            p+=2;
-            user = FALSE;
-
-            string_put(buf, "");
-        }
-        // 環境変数 $
-        else if(!squote && *p == '$') {
-            p++;
-            user = FALSE;
-
-            string_put(buf, "");
-        }
-
-        // 環境変数 @
-        else if(!squote && *p == '@' && *(p+1) == '@') {
-            p+=2;
-            user = FALSE;
-
-            string_put(buf, "");
-        }
-        // 環境変数 @@
-        else if(!squote && *p == '@') {
-            p++;
-            user = FALSE;
-
-            string_put(buf, "");
-        }
         /// シングルクォート、ダブルクォート中 ///
         else if(squote || dquote) {
             p++;
 
             string_put(buf, "");
         }
+        // リファレンス
+        else if(*p == '$') {
+            p++;
+            ref = TRUE;
+
+            string_put(buf, "");
+        }
         else if(*p == '~') {
             p++;
+            ref = FALSE;
             user = TRUE;
 
             string_put(buf, "");
@@ -1142,6 +1401,7 @@ char** readline_on_complete(const char* text, int start, int end)
         else if(*p == '*' || *p == '?' || *p =='[' && (*(p+1) != ' ' && *(p+1) != '\t' && *(p+1) == '\n')) 
         {
             p++;
+            ref = FALSE;
             user = FALSE;
 
             string_put(buf, "");
@@ -1160,11 +1420,13 @@ char** readline_on_complete(const char* text, int start, int end)
             }
 
             user = FALSE;
+            ref = FALSE;
         }
         else if(*p == '<' && program) {
             p++;
 
             program = FALSE;
+            ref = FALSE;
             user = FALSE;
 
             string_put(buf, "");
@@ -1180,6 +1442,7 @@ char** readline_on_complete(const char* text, int start, int end)
             program = FALSE;
             method = FALSE;
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1187,11 +1450,13 @@ char** readline_on_complete(const char* text, int start, int end)
         else if(*p == '{') {
             p++;
             program = TRUE;
+            string_put(command, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1203,10 +1468,12 @@ char** readline_on_complete(const char* text, int start, int end)
             p+=2;
 
             program = TRUE;
+            string_put(command, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
+            ref = FALSE;
             user = FALSE;
 
             string_put(buf, "");
@@ -1217,10 +1484,12 @@ char** readline_on_complete(const char* text, int start, int end)
             p++;
 
             program = TRUE;
+            string_put(command, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
+            ref = FALSE;
             user = FALSE;
 
             string_put(buf, "");
@@ -1233,11 +1502,13 @@ char** readline_on_complete(const char* text, int start, int end)
             p+=2;
 
             program = TRUE;
+            string_put(command, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1246,11 +1517,13 @@ char** readline_on_complete(const char* text, int start, int end)
             p+=2;
 
             program = TRUE;
+            string_put(command, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1259,11 +1532,13 @@ char** readline_on_complete(const char* text, int start, int end)
             p++;
 
             program = TRUE;
+            string_put(command, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1272,11 +1547,13 @@ char** readline_on_complete(const char* text, int start, int end)
             p++;
 
             program = TRUE;
+            string_put(command, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1285,11 +1562,13 @@ char** readline_on_complete(const char* text, int start, int end)
             p++;
 
             program = TRUE;
+            string_put(command, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
             user = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1301,6 +1580,7 @@ char** readline_on_complete(const char* text, int start, int end)
                 if(*p == '\n') {
                     p++;
                     program = TRUE;
+                    string_put(command, "");
                     break;
                 }
                 else {
@@ -1312,6 +1592,7 @@ char** readline_on_complete(const char* text, int start, int end)
             }
             user = FALSE;
             method = FALSE;
+            ref = FALSE;
 
             string_put(buf, "");
         }
@@ -1319,6 +1600,7 @@ char** readline_on_complete(const char* text, int start, int end)
             user = FALSE;
             program = FALSE;
             method = FALSE;
+            ref = FALSE;
             p++;
 
             string_put(buf, "");
@@ -1334,8 +1616,10 @@ char** readline_on_complete(const char* text, int start, int end)
         else if(*p == '(') {
             p++;
             program = TRUE;
+            string_put(command, "");
             method = FALSE;
             user = FALSE;
+            ref = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
@@ -1344,6 +1628,10 @@ char** readline_on_complete(const char* text, int start, int end)
         }
         else {
             string_push_back2(buf, *p);
+
+            if(program) {
+                string_push_back2(command, *p);
+            }
             p++;
         }
     }
@@ -1360,13 +1648,73 @@ char** readline_on_complete(const char* text, int start, int end)
         rl_completion_append_character= ' ';
         return rl_completion_matches(text, readline_method_completion);
     }
+    else if(ref) {
+        rl_completion_append_character= ' ';
+        return rl_completion_matches(text, readline_deref_completion);
+    }
 #if defined(HAVE_MIGEMO_H)
     else {
-        return rl_completion_matches(text, readline_file_completion_migemo);
+        if(strcmp(string_c_str(command), "sweep") == 0) {
+            return rl_completion_matches(text, readline_sweep_completion);
+        }
+        else if(strcmp(string_c_str(command), "ary") == 0 || strcmp(string_c_str(command), "ary_add") == 0 || strcmp(string_c_str(command), "ary_erase") == 0)
+        {
+            return rl_completion_matches(text, readline_ary_completion);
+        }
+        else if(strcmp(string_c_str(command), "hash") == 0 || strcmp(string_c_str(command), "hash_add") == 0 || strcmp(string_c_str(command), "hash_erase") == 0)
+        {
+            return rl_completion_matches(text, readline_hash_completion);
+        }
+        else if(strcmp(string_c_str(command), "global") == 0) {
+            return rl_completion_matches(text, readline_global_completion);
+        }
+        else if(strcmp(string_c_str(command), "class") == 0) {
+            return rl_completion_matches(text, readline_class_completion);
+        }
+        else if(strcmp(string_c_str(command), "def") == 0) {
+            return rl_completion_matches(text, readline_def_completion);
+        }
+        else if(strcmp(string_c_str(command), "object") == 0) {
+            return rl_completion_matches(text, readline_object_completion);
+        }
+        else if(strcmp(string_c_str(command), "ref") == 0) {
+            return rl_completion_matches(text, readline_ref_completion);
+        }
+        else {
+            return rl_completion_matches(text, readline_file_completion_migemo);
+        }
     }
 #else
     else {
-        return rl_completion_matches(text, readline_file_completion);
+        if(strcmp(string_c_str(command), "sweep") == 0) {
+            return rl_completion_matches(text, readline_sweep_completion);
+        }
+        else if(strcmp(string_c_str(command), "ary") == 0 || strcmp(string_c_str(command), "ary_add") == 0 || strcmp(string_c_str(command), "ary_erase") == 0)
+        {
+            return rl_completion_matches(text, readline_ary_completion);
+        }
+        else if(strcmp(string_c_str(command), "hash") == 0 || strcmp(string_c_str(command), "hash_add") == 0 || strcmp(string_c_str(command), "hash_erase") == 0)
+        {
+            return rl_completion_matches(text, readline_hash_completion);
+        }
+        else if(strcmp(string_c_str(command), "global") == 0) {
+            return rl_completion_matches(text, readline_global_completion);
+        }
+        else if(strcmp(string_c_str(command), "class") == 0) {
+            return rl_completion_matches(text, readline_class_completion);
+        }
+        else if(strcmp(string_c_str(command), "def") == 0) {
+            return rl_completion_matches(text, readline_def_completion);
+        }
+        else if(strcmp(string_c_str(command), "object") == 0) {
+            return rl_completion_matches(text, readline_object_completion);
+        }
+        else if(strcmp(string_c_str(command), "ref") == 0) {
+            return rl_completion_matches(text, readline_ref_completion);
+        }
+        else {
+            return rl_completion_matches(text, readline_file_completion);
+        }
     }
 #endif
 
