@@ -8,7 +8,7 @@ extern volatile BOOL gSigUser;
 
 /// ޥɤбʸ
 enum eCommandKind { 
-    kObject, kPointer, kVar, kGlobal, kExport, 
+    kObject, kRef, kVar, kGlobal, kExport, 
     kAry, kAryAdd, kAryErase, 
     kHash, kHashAdd, kHashErase,  
     kDef, kInherit, kSelf, kClass,
@@ -80,6 +80,8 @@ extern vector_obj* gStackFrame;
 extern vector_obj* gStackFramePointers;
 extern vector_obj* gStackFrameArrays;
 extern vector_obj* gStackFrameHashs;
+extern vector_obj* gStackFrameObjects;
+extern vector_obj* gStackFrameRefs;
     // ѿΥå
     // Ƥhash_obj*
     // Υϥѿ̾
@@ -198,19 +200,6 @@ void sWhile_delete(sWhile* self);
 void sWhile_save(sWhile* self, int fd);
 sWhile* sWhile_load(int fd);
 
-typedef struct
-{
-    sStatments* loops;
-    sStatments* loops2;
-} sTry;
-
-sTry* sTry_new(MANAGED sStatments* loops, MANAGED sStatments* loops2);
-sTry* sTry_new2(sTry* try_);
-void sTry_view(sTry* self);
-void sTry_delete(sTry* self);
-void sTry_save(sTry* self, int fd);
-sTry* sTry_load(int fd);
-
 typedef struct
 {
     sStatments* contents;
@@ -387,6 +376,30 @@ void sigchld_block(int block);
 
 extern hash_obj* gObjects;
 // ֥ ϥ֥̾ ƤsObject*
+//
+extern hash_obj* gRefs;
+// ֥ ϥ֥̾ ƤsObject*
+
+string_obj* sVar_new(char* str);
+string_obj* sGlobal_new(char* str);
+vector_obj* sAry_new(int first_size);
+hash_obj* sHash_new(int size);
+
+/// å
+void memchecker_init();
+enum eMemCheckerKind { kMCNothing, kMCRef, kMCArray, kMCObject, kMCHash, kMCVar, kMCGlobal, kMCFunction, kMCClass };
+
+void* memchecker_GC_malloc(int size, enum eMemCheckerKind obj_kind);
+    // åȤäGCΥΰ
+int memchecker_is_enable_mem(void* obj);
+    // ֥Ȥμ֤ 0:̵ 1:string_obj*
+
+typedef struct {
+    void* mMem;
+    enum eMemCheckerKind mKind;
+} sRef;
+
+sRef* sRef_new(void* mem, enum eMemCheckerKind kind);
 
 ////////////////////////////////////////////////////////////
 /// from saphire_commands.c
